import { useMemo } from "react";
import { getHoursInDayNumbers, getAvailableTasks, getLinesInDay, getSortedTasks } from "../../../../../helpers/CalendarHelpers";
import CalendarStandartCell from "../../../CalendarStandartCell/CalendarStandartCell";
import { useEffect, useRef, useState } from "react";
import { Grid } from "@mui/material";
import CalendarWeekTask from "./CalendarWeekTask/CalendarWeekTask";
import { getTasksWithInfoForPosition, getWidthLeftZIndex } from "./Helpers";



function CalendarColumnDayWeek({ hoursInDay, tasks, month, year, day, hourFormat, handleOpen, setCurrentTask, copyTask, createCopyTask, createTaskInCellHandler, modal }) {

  const [columnDaySize, setColumnDaySize] = useState({ width: 0, height: 0 })

  const dayColumnRef = useRef('')
  useEffect(() => {
    setColumnDaySize(prev => { return { height: dayColumnRef.current.offsetHeight, width: dayColumnRef.current.offsetWidth } })
  }, [hourFormat])
  
  const hours = useMemo(() => {
    return getHoursInDayNumbers(hoursInDay)
  }, [hoursInDay])

  const availableTasks = useMemo(() => {
    return getAvailableTasks(tasks, year, month, day)
  }, [tasks, month, year, day])


  const sortedTasks = useMemo(() => {
    return getSortedTasks(availableTasks)
  }, [availableTasks])


  const linesInDay = useMemo(() => {
    return getLinesInDay(availableTasks, sortedTasks, hoursInDay, hours, hourFormat)
  }, [availableTasks, hourFormat, hours, hoursInDay, sortedTasks])

  return (<>
    <Grid item xs={12 / 7} ref={dayColumnRef} sx={{ position: 'relative' }}>
      {linesInDay?.map((line, i) => {
        const boxes = getTasksWithInfoForPosition(line, sortedTasks, linesInDay, columnDaySize.height, hours)
        return (<>
          {boxes.map((task, index) => {
            const { width, left, zIndex } = getWidthLeftZIndex(task, columnDaySize.width, i)
            return (
              <CalendarWeekTask
                key={index}
                width={width}
                height={task.height}
                left={left}
                task={task.task}
                top={task.top}
                zIndex={zIndex}
                handleOpen={handleOpen}
                setCurrentTask={setCurrentTask}
                modal={modal}
              >
              </CalendarWeekTask>
            )
          })}
        </>)

      })}
      {hoursInDay?.map((hour, i) => {
        return (
          <CalendarStandartCell
            key={i}
            week={true}
            createTaskInCellHandler={createTaskInCellHandler}
            handleOpen={handleOpen}
            dayNumber={day}
            hours={hour}
            modal={modal}
          >
          </CalendarStandartCell>
        )
      })}
    </Grid>

  </>);
}

export default CalendarColumnDayWeek;